Functions implementing mutation genetic operator for GA-PARSIMONY. Method mutes a object@pmutation percentage of them. If the value corresponds to a model parameter, algorithm uses uniform random mutation. For binary select features, method sets to one if the random value between [0,1] is lower or equal to object@feat_mut_thres.
parsimony_mutation(object, ...)Return object with the population muted.
An object of class "ga_parsimony", usually resulting from a call to function ga_parsimony.
Further arguments passed to or from other methods.
Francisco Javier Martinez de Pison. fjmartin@unirioja.es. EDMANS Group. https://edmans.webs.com/
ga_parsimony