Functions for creating an initial population to be used in the GA-PARSIMONY process.
parsimony_population(object, type_ini_pop="randomLHS", ...)Return a matrix of dimension object@popSize rows and object@nParams+object@nFeatures columns.
An object of class "ga_parsimony", usually resulting from a call to function ga_parsimony.
How to create the initial population. 'random' optiom initialize a random population between the predefined ranges. Values 'randomLHS', 'geneticLHS', 'improvedLHS', 'maximinLHS' & 'optimumLHS' corresponds with several methods of the Latin Hypercube Sampling (see 'lhs' package for more details).
Further arguments passed to or from other methods.
Francisco Javier Martinez de Pison. fjmartin@unirioja.es. EDMANS Group. https://edmans.webs.com/
parsimony_population generates a random population of object@popSize individuals. For each individual a random chromosome is generated with object@nParams real values in the range [object@min_param, object@max_param] plus object@nFeatures random binary values for feature selection.
'random' or Latin Hypercube Sampling can be used to create a efficient spread initial population.
ga_parsimony