Summary method for class ga_parsimony-class.
# S4 method for ga_parsimony
summary(object, ...)
# S3 method for summary.ga_parsimony
print(x, digits = getOption("digits"), ...)The summary function returns an object of class ga_parsimony-class which can be printed by the corresponding print method. The function also returns invisibly a list with the information from the genetic algorithm search.
an object of class ga_parsimony-class.
an object of class summary.ga_parsimony.
number of significant digits.
further arguments passed to or from other methods.
Francisco Javier Martinez de Pison. fjmartin@unirioja.es. EDMANS Group. https://edmans.webs.com/
ga_parsimony